#include <kfiledialog.h>
#include <iostream.h>
#include <pi-macros.h>
#include <qlist.h>
#include <qlistbox.h>
#include <qfile.h>
#include <qpushbt.h>
#include <string.h>
#include <stdlib.h>
#include <kapp.h>
#include <kmsgbox.h>
#include "strToken.h"
#include "addressWidget.moc"
#include "pi-dlp.h"
#include "pi-address.h"
#include "kpilot.h"
#include "addressEditor.h"


AddressWidget::AddressWidget(KPilotInstaller* installer, QWidget* parent)
  : PilotComponent(parent), fTextWidget(0L)
    {
    setGeometry(0, 0, parent->geometry().width(), parent->geometry().height());
    setupWidget();
    fAddressList.setAutoDelete(true);
    installer->addComponentPage(this, klocale->translate("Address Book"));
    }

AddressWidget::~AddressWidget()
    {
    }

void
AddressWidget::initialize()
    {
    PilotDatabase* addressDB = KPilotLink::getPilotLink()->openLocalDatabase("AddressDB");
    int currentRecord = 0;
    PilotRecord* pilotRec;
    unsigned char buffer[0xffff];
    int appLen;
    PilotAddress* address;
    bool showSecrets;
    int i;

    KConfig* config = kapp->getConfig();
    config->setGroup(0L);
    showSecrets = (bool) config->readNumEntry("ShowSecrets");
    fCatList->clear();
    fAddressList.clear();
    if(addressDB->isDBOpen())
	{
	appLen = addressDB->readAppBlock(buffer, 0xffff);
	unpack_AddressAppInfo(&fAddressAppInfo, buffer, appLen);
	for(i = 0; i < 15; i++)
	    if(strlen(fAddressAppInfo.category.name[i]))
	      {
//  		cout << "Adding category: " << fAddressAppInfo.category.name[i] 
// 		     << " with ID: " << (int)fAddressAppInfo.category.ID[i] << endl;
		fCatList->insertItem(fAddressAppInfo.category.name[i]);
	      }

	while((pilotRec = addressDB->readRecordByIndex(currentRecord)) != 0L)
	  {
	    if(!(pilotRec->getAttrib() & dlpRecAttrDeleted))
	      if(!(pilotRec->getAttrib() & dlpRecAttrSecret) || showSecrets)
		{	    
		  address = new PilotAddress(pilotRec);
		  fAddressList.append(address);
		}
	    delete pilotRec;
	    currentRecord++;
	  }
	KPilotLink::getPilotLink()->closeDatabase(addressDB);
	}
    updateWidget();
    }

void
AddressWidget::preHotSync(char*)
{
}

void
AddressWidget::postHotSync()
{
  fAddressList.clear();
  initialize();
}

bool
AddressWidget::saveData()
    {
    return true;
    }


void
AddressWidget::setupWidget()
{
  fCatList = new QComboBox(this);
  fCatList->move(110, 25);
  connect(fCatList, SIGNAL(activated(int)), this, SLOT(slotSetCategory(int)));
  QLabel* label = new QLabel(klocale->translate("Addresses:"), this);
  label->move(10, 30);
  fListBox = new QListBox(this);
  fListBox->setGeometry(10, 60, 200, 150);
  connect(fListBox, SIGNAL(highlighted(int)), this, SLOT(slotShowAddress(int)));
  connect(fListBox, SIGNAL(selected(int)), this, SLOT(slotEditRecord()));
  label = new QLabel(klocale->translate("Address Info:"), this);
  label->move(290, 0);
  fTextWidget = new QMultiLineEdit(this, "textArea");
  fTextWidget->setGeometry(230, 30, 260, 290);
  fTextWidget->setFont(QFont("fixed", 10));
  fTextWidget->setReadOnly(TRUE);
  QPushButton* button = new QPushButton(klocale->translate("Edit Record"), this);
  button->move(10, 220);
  connect(button, SIGNAL(clicked()), this, SLOT(slotEditRecord()));
  button = new QPushButton(klocale->translate("New Record"), this);
  button->move(110, 220);
  connect(button, SIGNAL(clicked()), this, SLOT(slotCreateNewRecord()));
  button = new QPushButton(klocale->translate("Delete Record"), this);
  button->move(60, 250);
  connect(button, SIGNAL(clicked()), this, SLOT(slotDeleteRecord()));
  button = new QPushButton(klocale->translate("Import List"), this);
  button->move(10, 290);
  connect(button, SIGNAL(clicked()), this, SLOT(slotImportAddressList()));
  button = new QPushButton(klocale->translate("Export List"), this);
  button->move(110, 290);
  connect(button, SIGNAL(clicked()), this, SLOT(slotExportAddressList()));
}

void
AddressWidget::updateWidget()
{
  int listIndex = 0;
  int currentEntry = 0;
  char* title;
  int currentCatID = 0;

  // If a category is deleted after others have been added, none of the
  // category numbers are changed.  So we need to find the category number
  // for this category.
  while(strcmp(fAddressAppInfo.category.name[currentCatID], 
	       fCatList->text(fCatList->currentItem())) &&
	(currentCatID < fCatList->count()))
    currentCatID++;
  fListBox->clear();
  fAddressList.first();
  while(fAddressList.current())
    {
      if(fAddressList.current()->getCat() == currentCatID)
	{
	  // The list will delete it..
	  title = new char[255];
	  title[0] = 0L; // in case the first copy fails
	  if(fAddressList.current()->getField(entryLastname) &&
	     strcmp(fAddressList.current()->getField(entryLastname), ""))
	    strcpy(title, fAddressList.current()->getField(entryLastname));
	  if(fAddressList.current()->getField(entryFirstname) && 
	     strcmp(fAddressList.current()->getField(entryFirstname), ""))
	    {
	      if(title[0])
		strcat(title, ", ");
	      strcat(title, fAddressList.current()->getField(entryFirstname));
	    }
	  if(title[0] == 0L) // One last try
	    strcpy(title, fAddressList.current()->getField(entryCompany));
	  fListBox->insertItem(title, -1);
	  fLookupTable[currentEntry++] = listIndex;
	}
      listIndex++;
      fAddressList.next();
    }
    fTextWidget->clear();
}

void
AddressWidget::slotSetCategory(int)
{
  updateWidget();
}

void
AddressWidget::slotEditRecord()
  {
    int item = fListBox->currentItem();
    if(item == -1)
      return;
    item = fLookupTable[item];
    if(fAddressList.at(item)->id() == 0x0)
      {
	KMsgBox::message(0L, klocale->translate("Hot-Sync Required"), 
			 klocale->translate("Cannot edit new records until \r\n Hot-Synced with pilot."), 
			 KMsgBox::STOP);
	return;
      }
    AddressEditor* editor = new AddressEditor(fAddressList.at(item));
    connect(editor, SIGNAL(recordChangeComplete(PilotAddress*)),
	    this, SLOT(slotUpdateRecord(PilotAddress*)));
    editor->show();
  }

void
AddressWidget::slotCreateNewRecord()
{
  AddressEditor* editor = new AddressEditor(0L);
  connect(editor, SIGNAL(recordChangeComplete(PilotAddress*)),
	  this, SLOT(slotAddRecord(PilotAddress*)));
  editor->show();
}

void
AddressWidget::slotAddRecord(PilotAddress* address)
{
  int currentCatID = 0;

  // If a category is deleted after others have been added, none of the
  // category numbers are changed.  So we need to find the category number
  // for this category.
  while(strcmp(fAddressAppInfo.category.name[currentCatID], 
	       fCatList->text(fCatList->currentItem())))
    currentCatID++;
  address->setCat(currentCatID);
  fAddressList.append(address);
  writeAddress(address);
  updateWidget();
  fListBox->setCurrentItem(fAddressList.count() - 1); // Show the newest one
}

void
AddressWidget::slotUpdateRecord(PilotAddress* address)
{
  writeAddress(address);
  int currentRecord = fListBox->currentItem();
  updateWidget();
  fListBox->setCurrentItem(currentRecord);
}

void
AddressWidget::slotDeleteRecord()
{
  int item = fListBox->currentItem();
  if(item == -1)
    return;
  item = (int)fLookupTable[item];
  if(fAddressList.at(item)->id() == 0x0)
    {
      KMsgBox::message(0L, klocale->translate("Hot-Sync Required"), 
		       klocale->translate("Cannot delete new records until \r\n Hot-Synced with pilot."), 
		       KMsgBox::STOP);
      return;
    }
  if(KMsgBox::yesNo(0L, klocale->translate("Delete Record?"), 
		    klocale->translate("Delete currently selected record?")) == 2)
    return;
  PilotAddress* address = fAddressList.at(item);
  address->setAttrib(address->getAttrib() | dlpRecAttrDeleted);
  writeAddress(address);
  initialize();
}

void
AddressWidget::slotShowAddress(int which)
    {
    char text[0xffff];
    PilotAddress* theAdd = fAddressList.at(fLookupTable[which]);
    int i;
    int pad;
    text[0] = 0L;

    for(i = 0; i < 19; i++)
	{
	  if ((i > 2) && (i < 8))
	    {
	      strcat(text, fAddressAppInfo.phoneLabels[theAdd->getPhoneLabelIndex(i-3)]);
	      pad = 10 - strlen(fAddressAppInfo.labels[i]);
	    }
	  else
	    {
	      strcat(text, fAddressAppInfo.labels[i]);
	      pad = 10 - strlen(fAddressAppInfo.labels[i]);
	    }
 	strcat(text, ": ");
	while(pad > 0)
	    {
	    strcat(text, " ");
	    pad--;
	    }
	if(theAdd->getField(i))
	    strcat(text, theAdd->getField(i));
	if(i < 18)
	    strcat(text, "\n"); // So there's no return after note, it fits nicer.
	}
    fTextWidget->setText(text);
    }

void
AddressWidget::slotImportAddressList()
    {
    char importFormat[255];
    char delim[2];
    char nextField[255];
    bool useKeyField;

    KConfig* config = kapp->getConfig();
    PilotAddress* currentAddress;

    QString fileName = KFileDialog::getSaveFileName();
    if(fileName == 0L)
 	return;
    QFile inFile(fileName);
    if(inFile.open(IO_ReadOnly) == FALSE)
 	{
 	// show error!
 	return;
 	}
    QTextStream inputStream(&inFile);
    config->setGroup("Address Widget");
    useKeyField = config->readNumEntry("UseKeyField");
    QString nextRecord;

    while(inputStream.eof() == false)
	{
	strcpy(importFormat, config->readEntry("IncomingFormat"));
	delim[0] = importFormat[3];
 	delim[1] = 0L;

	
	StrTokenizer* formatTokenizer = new StrTokenizer(importFormat, delim);
	StrTokenizer* dataTokenizer;
	const char* nextToken = formatTokenizer->getNextField();

 	nextRecord = inputStream.readLine();
 	if(inputStream.eof() || (nextRecord == ""))
 	    break;
	dataTokenizer = new StrTokenizer(nextRecord, delim);
	strcpy(nextField, dataTokenizer->getNextField());
 	if(useKeyField)
	    currentAddress = findAddress(nextField, nextToken);
	else
	    currentAddress = 0L;
 	if(currentAddress == 0L)
 	    {
 	    currentAddress = new PilotAddress();
 	    fAddressList.append(currentAddress);
 	    }
 	setFieldBySymbol(currentAddress, nextToken, nextField);
  	while((nextToken = formatTokenizer->getNextField()) != NULL)
 	    {
	    strcpy(nextField, dataTokenizer->getNextField());
 	    setFieldBySymbol(currentAddress, nextToken, nextField);
 	    }
	delete formatTokenizer;
	delete dataTokenizer;
	writeAddress(currentAddress);
 	}
    inFile.close();
    updateWidget();
    }

PilotAddress*
AddressWidget::findAddress(const char* text, const char* symbol)
    {
    for(fAddressList.first() ; fAddressList.current(); fAddressList.next())
	if(strcmp(text, getFieldBySymbol(fAddressList.current(), symbol)) == 0)
	    return fAddressList.current();
    return 0L;
    }

void
AddressWidget::setFieldBySymbol(PilotAddress* rec, const char* symbol, const char* text)
    {
    if(strcasecmp(symbol, "%LN") == 0)
	rec->setField(entryLastname, text);
    else if(strcasecmp(symbol, "%FN") == 0)
	rec->setField(entryFirstname, text);
    else if(strcasecmp(symbol, "%CO") == 0)
	rec->setField(entryCompany, text);
    else if(strcasecmp(symbol, "%P1") == 0)
	rec->setField(entryPhone1, text);
    else if(strcasecmp(symbol, "%P2") == 0)
	rec->setField(entryPhone2, text);
    else if(strcasecmp(symbol, "%P3") == 0)
	rec->setField(entryPhone3, text);
    else if(strcasecmp(symbol, "%P4") == 0)
	rec->setField(entryPhone4, text);
    else if(strcasecmp(symbol, "%P5") == 0)
	rec->setField(entryPhone5, text);
    else if(strcasecmp(symbol, "%AD") == 0)
	rec->setField(entryAddress, text);
    else if(strcasecmp(symbol, "%CI") == 0)
	rec->setField(entryCity, text);
    else if(strcasecmp(symbol, "%ST") == 0)
	rec->setField(entryState, text);
    else if(strcasecmp(symbol, "%ZI") == 0)
	rec->setField(entryZip, text);
    else if(strcasecmp(symbol, "%CT") == 0)
	rec->setField(entryCountry, text);
    else if(strcasecmp(symbol, "%TI") == 0)
	rec->setField(entryTitle, text);
    else if(strcasecmp(symbol, "%C1") == 0)
	rec->setField(entryCustom1, text);
    else if(strcasecmp(symbol, "%C2") == 0)
	rec->setField(entryCustom2, text);
    else if(strcasecmp(symbol, "%C3") == 0)
	rec->setField(entryCustom3, text);
    else if(strcasecmp(symbol, "%C4") == 0)
	rec->setField(entryCustom4, text);
//     else if(strcasecmp(symbol, "%NO") == 0)
// 	rec->setField(entryNote, text);
    }

char*
AddressWidget::getFieldBySymbol(PilotAddress* rec, const char* symbol)
    {
    if(strcasecmp(symbol, "%LN") == 0)
	return rec->getField(entryLastname);
    else if(strcasecmp(symbol, "%FN") == 0)
	return rec->getField(entryFirstname);
    else if(strcasecmp(symbol, "%CO") == 0)
	return rec->getField(entryCompany);
    else if(strcasecmp(symbol, "%P1") == 0)
	return rec->getField(entryPhone1);
    else if(strcasecmp(symbol, "%P2") == 0)
	return rec->getField(entryPhone2);
    else if(strcasecmp(symbol, "%P3") == 0)
	return rec->getField(entryPhone3);
    else if(strcasecmp(symbol, "%P4") == 0)
	return rec->getField(entryPhone4);
    else if(strcasecmp(symbol, "%P5") == 0)
	return rec->getField(entryPhone5);
    else if(strcasecmp(symbol, "%AD") == 0)
	return rec->getField(entryAddress);
    else if(strcasecmp(symbol, "%CI") == 0)
	return rec->getField(entryCity);
    else if(strcasecmp(symbol, "%ST") == 0)
	return rec->getField(entryState);
    else if(strcasecmp(symbol, "%ZI") == 0)
	return rec->getField(entryZip);
    else if(strcasecmp(symbol, "%CT") == 0)
	return rec->getField(entryCountry);
    else if(strcasecmp(symbol, "%TI") == 0)
	return rec->getField(entryTitle);
    else if(strcasecmp(symbol, "%C1") == 0)
	return rec->getField(entryCustom1);
    else if(strcasecmp(symbol, "%C2") == 0)
	return rec->getField(entryCustom2);
    else if(strcasecmp(symbol, "%C3") == 0)
	return rec->getField(entryCustom3);
    else if(strcasecmp(symbol, "%C4") == 0)
	return rec->getField(entryCustom4);
//     else if(strcasecmp(symbol, "%NO") == 0)
// 	return rec->getField(entryNote);
    return 0L;
    }

void
AddressWidget::writeAddress(PilotAddress* which)
    {
    PilotDatabase* addressDB = KPilotLink::getPilotLink()->openLocalDatabase("AddressDB");
    PilotRecord* pilotRec = which->pack();
    addressDB->writeRecord(pilotRec);
    delete pilotRec;
    KPilotLink::getPilotLink()->closeDatabase(addressDB);
    }

void
AddressWidget::slotExportAddressList()
    {
    char exportFormat[255];
    char delim[2];
    char* nextToken;
    KConfig* config = kapp->getConfig();
    PilotAddress* currentAddress;

    QString fileName = KFileDialog::getSaveFileName();
    if(fileName == 0L)
 	return;
    QFile outFile(fileName);
    if(outFile.open(IO_WriteOnly | IO_Truncate) == FALSE)
 	{
 	// show error!
 	return;
 	}
    QTextStream outStream(&outFile);
    config->setGroup("Address Widget");
      
    for(currentAddress = fAddressList.first(); currentAddress; currentAddress = fAddressList.next())
	{
	strcpy(exportFormat, config->readEntry("OutgoingFormat"));
	delim[0] = exportFormat[3];
	delim[1] = 0L;

	nextToken = strtok(exportFormat, delim);
	outStream << getFieldBySymbol(currentAddress, nextToken);
	
	while((nextToken = strtok(NULL, delim)) != NULL)
	    {
	    outStream << delim;
	    outStream << getFieldBySymbol(currentAddress, nextToken);
	    }
	outStream << "\n";
	}
    outFile.close();
    }
