#include <sys/types.h>
#include <dirent.h>
#include <iostream.h>
#include <fstream.h>
#include <qfile.h>
#include <qlist.h>
#include <qstring.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <kurl.h>
#include <kmsgbox.h>
#include <kstatusbar.h>
#include <qlistbox.h>
#include <kwm.h>
#include <kpilotOptions.h>
#include <kprocess.h>
#include "kpilot.moc"
#include "kpilotlink.h"
#include "messageDialog.h"
#include "addressWidget.h"
#include "kpilot_on_pp.h"
#include "hotsync.h"
#include "statusMessages.h"
#include "conduitSetup.h"

const int KPilotInstaller::ID_FILE_QUIT = 1;
const int KPilotInstaller::ID_FILE_SETTINGS = 2;
const int KPilotInstaller::ID_FILE_BACKUP = 3;
const int KPilotInstaller::ID_FILE_RESTORE = 4;
const int KPilotInstaller::ID_HELP_HELP = 5;
const int KPilotInstaller::ID_HELP_ABOUT = 6;
const int KPilotInstaller::ID_CONDUITS_ENABLE = 7;
const int KPilotInstaller::ID_CONDUITS_SETUP = 8;
const int KPilotInstaller::ID_COMBO = 3;

KPilotInstaller::KPilotInstaller()
  : KTopLevelWidget(), fMenuBar(0L), fStatusBar(0L), fToolBar(0L), 
    fQuitAfterCopyComplete(false), fManagingWidget(0L), fPilotLink(0L),
    fPilotCommandSocket(0L), fPilotStatusSocket(0L), fKillDaemonOnExit(false)
    {
    KConfig* config = kapp->getConfig();
    if(config->readNumEntry("Configured", 0) != 3)
	{
	// If we haven't been configured, force the user to set things up.
	config->writeEntry("Configured", 3);
	KPilotOptions* options = new KPilotOptions(this);
	options->show();
	delete options;
	}
    if(config->readNumEntry("NextUniqueID", 0) == 0)
      {
	// Is this an ok value to use??
	config->writeEntry("NextUniqueID", 0xf000);
	config->sync();
      }
    initPilotLink();
    setupWidget();
    initComponents();
    initStatusLink();  // This is separate to allow components to initialize
    slotModeSelected(0);
    show();
    }

KPilotInstaller::~KPilotInstaller()
    {
      delete fPilotStatusSocket;
      if(fKillDaemonOnExit)
	{
	  ofstream out(fPilotCommandSocket->socket());
	  out << -3 << endl;
	}
//      if(fMenuBar)
//    	delete fMenuBar;
//      if(fStatusBar)
//    	delete fStatusBar;
//      if(fToolBar)
//    	delete fToolBar;
    }

void
KPilotInstaller::setupWidget()
    {
    KWM::setIcon(winId(), kapp->getIcon());
    setCaption(klocale->translate("KPilot v3.1"));
    setMinimumSize(500,405);
    setMaximumSize(500,405);
    initMenu();
    initStatusBar();
    initToolBar();
    fManagingWidget = new QWidget(this);
    fManagingWidget->setMinimumSize(500,330);
    fManagingWidget->show();
    setView(fManagingWidget, FALSE);
    }

void
KPilotInstaller::initComponents()
    {
    PilotComponent* aComponent;
    
    QLabel* titleScreen = new QLabel(getManagingWidget());
    titleScreen->setPixmap(QPixmap(kpilot_on_pp));
    titleScreen->setAlignment(AlignCenter);
    titleScreen->setBackgroundColor(QColor("black"));
    titleScreen->setGeometry(0, 0, getManagingWidget()->geometry().width(), 
			           getManagingWidget()->geometry().height());
    fVisibleWidgetList.append(titleScreen);
    
    aComponent = new MemoWidget(this, getManagingWidget());
    aComponent->initialize();
    fPilotComponentList.append(aComponent);
    aComponent = new AddressWidget(this, getManagingWidget());
    aComponent->initialize();
    fPilotComponentList.append(aComponent);
    aComponent = new FileInstallWidget(this, getManagingWidget());
    aComponent->initialize();
    fPilotComponentList.append(aComponent);
    }

void 
KPilotInstaller::initStatusBar()
    {
    fStatusBar = new KStatusBar(this, "statusbar");
    fStatusBar->insertItem(klocale->translate("Welcome to KPilot v3.1"), 0);
    fStatusBar->show();
    setStatusBar(fStatusBar);
    }

void
KPilotInstaller::initToolBar()
  {
  fToolBar = new KToolBar(this, "toolbar");
  QPixmap icon(hotsync_icon);

  fToolBar->insertButton(icon, 0, SIGNAL(clicked()), this, SLOT(doHotSync()),
			 TRUE, "Hot-Sync");
  icon.load(kapp->kde_toolbardir() + "/exit.xpm");
  
  fToolBar->insertButton( icon, 0, SIGNAL(clicked()), this, SLOT(quit()),
			  TRUE, "Quit");
  fToolBar->insertCombo(klocale->translate("Pilot Application"), KPilotInstaller::ID_COMBO, false, SIGNAL(activated(int)), this, 
  			SLOT(slotModeSelected(int)), true, klocale->translate("KPilot Mode"), 140, 0);

  // This only works after Beta 4.. 
  fToolBar->alignItemRight(KPilotInstaller::ID_COMBO);
  addToolBar(fToolBar);
  }


void
KPilotInstaller::slotModeSelected(int selected)
    {
      if((unsigned int)selected >= fVisibleWidgetList.count())
	{
	  cout << "KPilotInstaller::slotModeSelected() - NO SUCH COMPONENT!!" << endl;
	  return;
	}
      for(fVisibleWidgetList.first(); fVisibleWidgetList.current(); fVisibleWidgetList.next())
 	fVisibleWidgetList.current()->hide();
      fVisibleWidgetList.at(selected)->show();
    }

void
KPilotInstaller::initPilotLink()
    {
      if(fPilotLink)
 	{
 	cout << "initPilotLink: Pilot Link already created??" << endl;
 	return;
 	}
      fPilotLink = new KPilotLink();
      KConfig* config = kapp->getConfig();
      config->setGroup(0L);
      if(fPilotCommandSocket == 0L)
	{
	  fPilotCommandSocket = new KSocket("localhost", 31509);
	  if(fPilotCommandSocket->socket() < 0)
	    {
	      // It wasn't running...
	      MessageDialog* messageDialog = new MessageDialog(klocale->translate("KPilot v3.1"));
	      messageDialog->setMessage(klocale->translate("Starting Sync Daemon.  Please Wait."));
	      messageDialog->show();
	      kapp->processEvents();
	      delete fPilotCommandSocket;
	      KProcess pilotDaemon;
	      pilotDaemon << "kpilotDaemon";
	      pilotDaemon.start(KProcess::DontCare);
	      sleep(4);
	      fPilotCommandSocket = new KSocket("localhost", 31509);
	      if(fPilotCommandSocket->socket() < 0)
		{
		  KMsgBox::message(this, klocale->translate("Cannot connect to Daemon"), 
				   klocale->translate("Cannot start KPilot Daemon.  Check settings."), KMsgBox::STOP);
		  KPilotOptions* options = new KPilotOptions(this);
		  options->show(); // It's modal..
		  delete options;
		  if(KMsgBox::yesNo(0L, klocale->translate("Restart?"), 
				    klocale->translate("Restart KPilot?")) == 2)
		    exit(1);
		  else
		    {
		      execl("kpilot", "kpilot", 0);
		      exit(1);
		    }
		}
	      fKillDaemonOnExit = true;
	      messageDialog->hide();
	      delete messageDialog;
	    }
 	  fLinkCommand[0] = 0L;
	}
      else // We were called after a reconfigure
	{
	  ofstream out(fPilotCommandSocket->socket());
	  out << "-2" << endl;
	}
    }    
    
void
KPilotInstaller::initStatusLink()
{
  fPilotStatusSocket = new KSocket("localhost", 31510);
  connect(fPilotStatusSocket, SIGNAL(readEvent(KSocket*)),
	  this, SLOT(slotDaemonStatus(KSocket*)));
  fPilotStatusSocket->enableRead(true);
}

void
KPilotInstaller::slotDaemonStatus(KSocket* daemon)
{
  ifstream in(daemon->socket());
  int status;
  
  in.read(&status, sizeof(int));
  if(status == CStatusMessages::SYNC_STARTING)
    {
      fLastWidgetSelected = fToolBar->getCombo(KPilotInstaller::ID_COMBO)->currentItem();
      fToolBar->getCombo(KPilotInstaller::ID_COMBO)->setCurrentItem(0);
      slotModeSelected(0);
      fToolBar->getCombo(KPilotInstaller::ID_COMBO)->setEnabled(false);
      if(fLinkCommand[0] == 0L)
	doHotSync();
      fStatusBar->changeItem(klocale->translate("Hot-Sync in progress..."), 0);
      ofstream out(fPilotCommandSocket->socket());
      out << fLinkCommand << flush;
    }
  else if(status == CStatusMessages::SYNC_COMPLETED)
    {
      fToolBar->getCombo(KPilotInstaller::ID_COMBO)->setEnabled(true);
      fToolBar->getCombo(KPilotInstaller::ID_COMBO)->setCurrentItem(fLastWidgetSelected);
      slotModeSelected(fLastWidgetSelected);
      fStatusBar->changeItem(klocale->translate("Hot-Sync complete."), 0);
      fLinkCommand[0] = 0L;
      for(fPilotComponentList.first(); fPilotComponentList.current(); fPilotComponentList.next())
	//fPilotComponentList.current()->postHotSync();
	fPilotComponentList.current()->initialize();
    }
  else if(status == CStatusMessages::FILE_INSTALL_REQUEST)
    {
      for(fPilotComponentList.first(); fPilotComponentList.current(); fPilotComponentList.next())
	fPilotComponentList.current()->initialize();
    }
}

void
KPilotInstaller::doBackup()
{
  fStatusBar->changeItem(klocale->translate("Backing up pilot. Please press the hot-sync button."), 0);
  sprintf(fLinkCommand, "%d\n", KPilotLink::Backup);
}

void
KPilotInstaller::doRestore()
{
  fStatusBar->changeItem(klocale->translate("Restoring pilot. Please press the hot-sync button."), 0);
  sprintf(fLinkCommand, "%d\n", KPilotLink::Restore);
}
  
void
KPilotInstaller::doHotSync()
{
  fStatusBar->changeItem(klocale->translate("Hot-Syncing.  Please press the hot-sync button."), 0);
  if (fLinkCommand[0] == 0)
    {
      sprintf(fLinkCommand, "%d\n", KPilotLink::HotSync);
      for(fPilotComponentList.first(); fPilotComponentList.current(); fPilotComponentList.next())
	fPilotComponentList.current()->preHotSync(fLinkCommand);
    }
}

void
KPilotInstaller::closeEvent(QCloseEvent *e)
{
  quit();
  e->accept();
}

void
KPilotInstaller::initMenu()
    {
    QPopupMenu* fileMenu = new QPopupMenu;
    fileMenu->insertItem(klocale->translate("&Settings"), KPilotInstaller::ID_FILE_SETTINGS);
    fileMenu->insertSeparator(-1);
    fileMenu->insertItem(klocale->translate("&Backup"), KPilotInstaller::ID_FILE_BACKUP);
    fileMenu->insertItem(klocale->translate("&Restore"), KPilotInstaller::ID_FILE_RESTORE);
    fileMenu->insertSeparator(-1);
    fileMenu->insertItem(klocale->translate("&Quit"), KPilotInstaller::ID_FILE_QUIT);
    connect(fileMenu, SIGNAL (activated(int)), SLOT (menuCallback(int)));
    
    QPopupMenu* conduitMenu = new QPopupMenu;
    conduitMenu->insertItem(klocale->translate("&Setup"), KPilotInstaller::ID_CONDUITS_SETUP);
    connect(conduitMenu, SIGNAL(activated(int)), SLOT(menuCallback(int)));

    QPopupMenu* helpMenu = new QPopupMenu;
    helpMenu->insertItem(klocale->translate("&About"), ID_HELP_ABOUT);
    helpMenu->insertItem(klocale->translate("&Contents"), ID_HELP_HELP);
    connect(helpMenu, SIGNAL(activated(int)), SLOT (menuCallback(int)));
    
    this->fMenuBar = new KMenuBar(this);
    this->fMenuBar->insertItem(klocale->translate("&File"), fileMenu);
    this->fMenuBar->insertItem(klocale->translate("&Conduits"), conduitMenu);
    this->fMenuBar->insertSeparator();
    this->fMenuBar->insertItem(klocale->translate("&Help"), helpMenu);
    this->fMenuBar->show();
    setMenu(this->fMenuBar);
    }

void
KPilotInstaller::fileInstalled(int )
    {
    }

void
KPilotInstaller::quit()
    {
    for(fPilotComponentList.first(); fPilotComponentList.current(); fPilotComponentList.next())
	fPilotComponentList.current()->saveData();
    kapp->quit();
    }

// Adds 'name' to the pull down menu of components
void
KPilotInstaller::addComponentPage(QWidget* widget, QString name)
    {
    fVisibleWidgetList.append(widget);
    fToolBar->insertComboItem(KPilotInstaller::ID_COMBO, name,
			      fVisibleWidgetList.count()-1);
    }

void
KPilotInstaller::menuCallback(int item)
  {
  KPilotOptions* options = 0L;
  CConduitSetup* conSetup = 0L;

  switch(item)
      {
      case KPilotInstaller::ID_HELP_ABOUT:
	KMsgBox::message(0L, klocale->translate("KPilot v3.1"), 
			 klocale->translate("KPilot v3.1 Hot-Sync Software for Unix\nBy: Dan Pilone\nEmail: pilone@slac.com"), 
			 KMsgBox::INFORMATION);
	break;
      case KPilotInstaller::ID_HELP_HELP:
	kapp->invokeHTMLHelp("kpilot/index.html", "");
	break;
      case KPilotInstaller::ID_FILE_QUIT:
	quit();
	break;
       case KPilotInstaller::ID_FILE_SETTINGS:
	 options = new KPilotOptions(this);
	 options->show(); // It's modal..
	 delete options;
	 destroyPilotLink(); // Get rid of the old one
	 initPilotLink(); // make a new one..
	 for(fPilotComponentList.first(); fPilotComponentList.current(); fPilotComponentList.next())
	   //fPilotComponentList.current()->postHotSync();
	   fPilotComponentList.current()->initialize();
	 break;
      case KPilotInstaller::ID_FILE_BACKUP:
	doBackup();
 	break;
      case KPilotInstaller::ID_FILE_RESTORE:
	doRestore();
	break;
      case KPilotInstaller::ID_CONDUITS_SETUP:
	conSetup = new CConduitSetup(this);
	conSetup->show();
	break;
      }
  }

void 
KPilotInstaller::slotSyncDone(KProcess*)
{
  fStatusBar->changeItem(klocale->translate("Updating display..."), 0);
  kapp->processEvents();
  for(fPilotComponentList.first(); fPilotComponentList.current(); fPilotComponentList.next())
    fPilotComponentList.current()->postHotSync();
  fStatusBar->changeItem(klocale->translate("Hot-Sync complete."),0);
}

void 
KPilotInstaller::testDir(QString name)
    {
    DIR *dp;
    dp = opendir(name);
    if(dp == 0L)
	::mkdir (name, S_IRWXU);
    else
	closedir( dp );
    }
    
int main(int argc, char** argv)
    {
    QStrList fileList;
    KPilotInstaller* tp;
    KApplication a(argc, argv, "kpilot");
    tp = new KPilotInstaller();
    a.setMainWidget(tp);
    return a.exec();
    }

