#include <qlabel.h>
#include <kapp.h>
#include <kconfig.h>
#include <stdlib.h>
#include <kwm.h>
#include <kfm.h>
#include <kmsgbox.h>
#include "kpilotOptions.moc"

KPilotOptions::KPilotOptions(QWidget* parent)
  : QTabDialog(parent, "optionsDialog", true)
    {
    resize(400, 400);
    setCancelButton();
    setupWidget();
    setCaption(klocale->translate("KPilot Options"));
    connect(this, SIGNAL(applyButtonPressed()), this, SLOT(commitChanges()));
    connect(this, SIGNAL(cancelButtonPressed()), this, SLOT(cancelChanges()));
    }

KPilotOptions::~KPilotOptions()
    {
    }

void
KPilotOptions::commitChanges()
    {
    KConfig* config = kapp->getConfig();
    config->setGroup(0L);
    config->writeEntry("PilotDevice", fPilotDevice->text());
    config->writeEntry("PilotSpeed", fPilotSpeed->currentItem());
    config->writeEntry("UserName", fUserName->text());
    config->writeEntry("SyncFiles", (int)fSyncFiles->isChecked());
    config->writeEntry("OverwriteRemote", (int)fOverwriteRemote->isChecked());
    config->writeEntry("StartDaemonAtLogin", (int)fStartDaemonAtLogin->isChecked());
    config->writeEntry("StartKPilotAtHotSync", (int)fStartKPilotAtHotSync->isChecked());
    config->writeEntry("DockDaemon", (int)fDockDaemon->isChecked());
    config->setGroup("Address Widget");
    config->writeEntry("IncomingFormat", fIncomingFormat->text());
    config->writeEntry("OutgoingFormat", fOutgoingFormat->text());
    config->writeEntry("UseKeyField", (int)fUseKeyField->isChecked());
    config->sync(); // So that the daemon can get at the changes.
    }

void
KPilotOptions::cancelChanges()
    {
    }

void
KPilotOptions::slotSetupDaemon()
{
  QString destDir = getenv("HOME");
  
  if(fStartDaemonAtLogin->isChecked())
    {
      fKFM = new KFM;
      if(fKFM->isOK() == false)
	{
	  KMsgBox::message(0L, klocale->translate("Setup Daemon"), 
			   klocale->translate("Cannot connect to KFM."), 
			   KMsgBox::STOP);
	  delete fKFM;
	  fKFM = 0L;
	  return;
	}
      connect(fKFM, SIGNAL(finished()), this, SLOT(slotKFMFinished()));
      QString destDir = getenv("HOME");
      fKFM->copy(kapp->kde_appsdir() + "/Utilities/KPilotDaemon.kdelnk", destDir + "/Desktop/Autostart/KPilotDaemon.kdelnk");
      // memory leak..
    }
  else
      {
	unlink(destDir + "/Desktop/Autostart/KPilotDaemon.kdelnk");
      }
}

void KPilotOptions::slotKFMFinished()
{
  delete fKFM;
  fKFM = 0L;
}

void
KPilotOptions::setupWidget()
    {
    KConfig* config = kapp->getConfig();
    QWidget* currentWidget;
    QLabel* currentLabel;

    config->setGroup(0L);
    currentWidget = new QWidget(this);
    currentLabel = new QLabel(klocale->translate("Pilot Device: "), currentWidget);
    currentLabel->adjustSize();
    currentLabel->move(10, 12);
    fPilotDevice = new QLineEdit(currentWidget);
    fPilotDevice->setText(config->readEntry("PilotDevice", "/dev/pilot"));
    fPilotDevice->resize(100, fPilotDevice->height());
    fPilotDevice->move(110,10);
    
    currentLabel = new QLabel(klocale->translate("Speed: "), currentWidget);
    currentLabel->adjustSize();
    currentLabel->move(220, 12);
    fPilotSpeed = new QComboBox(currentWidget);
    fPilotSpeed->insertItem("9600");
    fPilotSpeed->insertItem("19200");
    fPilotSpeed->insertItem("38400");
    fPilotSpeed->insertItem("57600");
    fPilotSpeed->insertItem("115200");
    fPilotSpeed->setCurrentItem(config->readNumEntry("PilotSpeed", 0));
    fPilotSpeed->move(280, 10);

    currentLabel = new QLabel(klocale->translate("Pilot User: "), currentWidget);
    currentLabel->adjustSize();
    currentLabel->move(10, 52);
    fUserName = new QLineEdit(currentWidget);
    fUserName->setText(config->readEntry("UserName", "$USER"));
    fUserName->resize(200, fUserName->height());
    fUserName->move(110, 50);

    currentLabel = new QLabel(klocale->translate("Sync Options:"), currentWidget);
    currentLabel->adjustSize();
    currentLabel->move(10, 120);

    fSyncFiles = new QCheckBox(klocale->translate("Sync &Files"), currentWidget);
    fSyncFiles->adjustSize();
    fSyncFiles->setChecked(config->readNumEntry("SyncFiles", 1));
    fSyncFiles->move(110, 120);

    fOverwriteRemote = new QCheckBox(klocale->translate("Local &overrides Pilot."), currentWidget);
    fOverwriteRemote->adjustSize();
    fOverwriteRemote->setChecked(config->readNumEntry("OverwriteRemote", 0));
    fOverwriteRemote->move(110, 140);

    fStartDaemonAtLogin = new QCheckBox(klocale->translate("Start Hot-Sync Daemon at login. (Only available with KFM)"), 
					currentWidget);
    fStartDaemonAtLogin->adjustSize();
    fStartDaemonAtLogin->setChecked(config->readNumEntry("StartDaemonAtLogin", 0));
    fStartDaemonAtLogin->move(10, 180);
    KFM* kfm = new KFM();
    if(kfm->isKFMRunning() != true)
      fStartDaemonAtLogin->setEnabled(false);
    delete kfm;
    connect(fStartDaemonAtLogin, SIGNAL(clicked()), this, SLOT(slotSetupDaemon()));

    fStartKPilotAtHotSync = new QCheckBox(klocale->translate("Start KPilot at Hot-Sync."), currentWidget);
    fStartKPilotAtHotSync->adjustSize();
    fStartKPilotAtHotSync->setChecked(config->readNumEntry("StartKPilotAtHotSync", 0));
    fStartKPilotAtHotSync->move(10, 200);

    fDockDaemon = new QCheckBox(klocale->translate("Show Daemon in KPanel. (Only available with KWM.)"), 
				currentWidget);
    fDockDaemon->adjustSize();
    fDockDaemon->setChecked(config->readNumEntry("DockDaemon", 0));
    KWM* kwm = new KWM();
    if(kwm->isKWMInitialized() == false)
      fDockDaemon->setEnabled(false);
    delete kwm;
    fDockDaemon->move(10, 220);
    


    addTab(currentWidget, klocale->translate("&General"));

    currentWidget = new QWidget(this);
    config->setGroup("Address Widget");
    
    currentLabel = new QLabel(klocale->translate("Import Format:"), currentWidget);
    currentLabel->adjustSize();
    currentLabel->move(10, 20);

    fIncomingFormat = new QLineEdit(currentWidget);
    fIncomingFormat->setText(config->readEntry("IncomingFormat", "%LN,%FN,%CO,%P1,%P2,%P3,%P4,%P5,%AD,%CI,%ST,%ZI,%CT,%TI,%C1,%C2,%C3,%C4"));
    fIncomingFormat->resize(250, fIncomingFormat->height());
    fIncomingFormat->move(110, 10);

    currentLabel = new QLabel(klocale->translate("Export Format:"), currentWidget);
    currentLabel->adjustSize();
    currentLabel->move(10, 55);

    fOutgoingFormat = new QLineEdit(currentWidget);
    fOutgoingFormat->setText(config->readEntry("OutgoingFormat", "%LN,%FN,%CO,%P1,%P2,%P3,%P4,%P5,%AD,%CI,%ST,%ZI,%CT,%TI,%C1,%C2,%C3,%C4"));
    fOutgoingFormat->resize(250, fIncomingFormat->height());
    fOutgoingFormat->move(110, 50);

    currentLabel = new QLabel(klocale->translate("%FN = First Name"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 100);
    
    currentLabel = new QLabel(klocale->translate("%LN = Last Name"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 120);
    
    currentLabel = new QLabel(klocale->translate("%CO = Company"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 140);
    
    currentLabel = new QLabel(klocale->translate("%P1 = Phone Number 1"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 160);
    
    currentLabel = new QLabel(klocale->translate("%P2 = Phone Number 2"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 180);

    currentLabel = new QLabel(klocale->translate("%P3 = Phone Number 3"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 200);

    currentLabel = new QLabel(klocale->translate("%P4 = Phone Number 4"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 220);

    currentLabel = new QLabel(klocale->translate("%P5 = Phone 5 (E-mail)"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 240);

    currentLabel = new QLabel(klocale->translate("%AD = Address"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 260);

    currentLabel = new QLabel(klocale->translate("%CI = City"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(25, 280);

    currentLabel = new QLabel(klocale->translate("%ST = State"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 100);

    currentLabel = new QLabel(klocale->translate("%ZI = Zip Code"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 120);

    currentLabel = new QLabel(klocale->translate("%CT = Country"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 140);

    currentLabel = new QLabel(klocale->translate("%TI = Title"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 160);

    currentLabel = new QLabel(klocale->translate("%C1 = Custom 1"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 180);

    currentLabel = new QLabel(klocale->translate("%C2 = Custom 2"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 200);

    currentLabel = new QLabel(klocale->translate("%C3 = Custom 3"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 220);
    
    currentLabel = new QLabel(klocale->translate("%C4 = Custom 4"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 240);
    
//     currentLabel = new QLabel("%NO = Note", currentWidget);
//     currentLabel->setFont(QFont("fixed", 10));
//     currentLabel->adjustSize();
//     currentLabel->move(225, 260);

    currentLabel = new QLabel(klocale->translate("%IG = Ignore (used\n          for import)"), currentWidget);
    currentLabel->setFont(QFont("fixed", 10));
    currentLabel->adjustSize();
    currentLabel->move(225, 260);
    
    fUseKeyField = new QCheckBox(klocale->translate("Use &Key Field"), currentWidget);
    fUseKeyField->adjustSize();
    fUseKeyField->setChecked(config->readNumEntry("UseKeyField", 0));
    fUseKeyField->move(150,300);
			     
    addTab(currentWidget, klocale->translate("A&ddress Settings"));
    }
