#include <time.h>
#include <iostream.h>
#include <pi-macros.h>
#include <qdir.h>
#include <qlist.h>
#include <qlistbox.h>
#include <qfile.h>
#include <qpushbt.h>
#include <kapp.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include "kpilot.h"
#include "memoWidget.moc"
#include "pi-dlp.h"

int MemoWidget::MAX_MEMO_LEN = 8192;

MemoWidget::MemoWidget(KPilotInstaller* installer, QWidget* parent)
  : PilotComponent(parent), fTextWidget(0L)
    {
    setGeometry(0, 0, parent->geometry().width(), parent->geometry().height());
    setupWidget();
    fMemoList.setAutoDelete(true);
    installer->addComponentPage(this, "Memos");
    }

MemoWidget::~MemoWidget()
    {
    }

void
MemoWidget::initialize()
    {
    PilotDatabase* memoDB = KPilotLink::getPilotLink()->openLocalDatabase("MemoDB");
    int currentRecord = 0;
    PilotRecord* pilotRec;
    bool showSecrets;
    PilotMemo* memo;
    int appLen;
    unsigned char buffer[0xffff];
    int i;

    KConfig* config = kapp->getConfig();
    config->setGroup(0L);
    showSecrets = (bool) config->readNumEntry("ShowSecrets");
    fCatList->clear();
    fMemoList.clear();
    if(memoDB->isDBOpen())
	{
	  appLen = memoDB->readAppBlock(buffer, 0xffff);
	  unpack_MemoAppInfo(&fMemoAppInfo, buffer, appLen);
	  for(i = 0; i < 15; i++)
	    if(strlen(fMemoAppInfo.category.name[i]))
	      {
		fCatList->insertItem(fMemoAppInfo.category.name[i]);
	      }
	while((pilotRec = memoDB->readRecordByIndex(currentRecord)) != 0L)
	    {
	      if(!(pilotRec->getAttrib() & dlpRecAttrDeleted))
		if(!(pilotRec->getAttrib() & dlpRecAttrSecret) || showSecrets)
		  {
		    memo = new PilotMemo(pilotRec);
		    fMemoList.append(memo);
		  }
	      delete pilotRec;
	      currentRecord++;
	    }
	KPilotLink::getPilotLink()->closeDatabase(memoDB);
	}
    updateWidget();
    }

void
MemoWidget::preHotSync(char*)
{
}

void
MemoWidget::postHotSync()
{
  fMemoList.clear();
  initialize();
}

bool
MemoWidget::saveData()
    {
    return true;
    }


void
MemoWidget::setupWidget()
{
  fCatList = new QComboBox(this);
  fCatList->move(110, 25);
  connect(fCatList, SIGNAL(activated(int)), this, SLOT(slotSetCategory(int)));
  QLabel* label = new QLabel(klocale->translate("Memos:"), this);
  label->move(10, 30);
  fListBox = new QListBox(this);
  fListBox->setGeometry(10, 60, 200, 150);
  connect(fListBox, SIGNAL(highlighted(int)), this, SLOT(slotShowMemo(int)));
  label = new QLabel(klocale->translate("Memo Text:"), this);
  label->move(290, 0);
  fTextWidget = new QMultiLineEdit(this, "textArea");
  fTextWidget->setGeometry(230, 30, 260, 290);
  connect(fTextWidget, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
  QPushButton* button = new QPushButton(klocale->translate("Import Memo"), this);
  button->move(10, 220);
  connect(button, SIGNAL(clicked()), this, SLOT(slotImportMemo()));
  button = new QPushButton(klocale->translate("Export Memo"), this);
  button->move(110, 220);
  connect(button, SIGNAL(clicked()), this, SLOT(slotExportMemo()));
  button = new QPushButton(klocale->translate("Delete Memo"), this);
  button->move(60, 250);
  connect(button, SIGNAL(clicked()), this, SLOT(slotDeleteMemo()));
}

void 
MemoWidget::slotSetCategory(int)
{
  updateWidget();
}

void
MemoWidget::slotDeleteMemo()
{
  int item = fListBox->currentItem();
  if(item == -1)
    return;
  item = fLookupTable[item];
  if(fMemoList.at(item)->id() == 0x0)
    {
      KMsgBox::message(0L, klocale->translate("Hot-Sync Required"), 
		       klocale->translate("Cannot delete new memo until \r\n Hot-Synced with pilot."), 
		       KMsgBox::STOP);
      return;
    }
  if(KMsgBox::yesNo(0L, klocale->translate("Delete Memo?"), 
		    klocale->translate("Delete currently selected memo?")) == 2)
    return;
  PilotMemo* memo = fMemoList.at(item);
  memo->setAttrib(memo->getAttrib() | dlpRecAttrDeleted);
  writeMemo(memo);
  initialize();
}

void
MemoWidget::updateWidget()
{
  int listIndex = 0;
  int currentEntry = 0;
  int currentCatID = 0;
  
  // If a category is deleted after others have been added, none of the
  // category numbers are changed.  So we need to find the category number
  // for this category.
  while(strcmp(fMemoAppInfo.category.name[currentCatID], 
	       fCatList->text(fCatList->currentItem())) && 
	(currentCatID < fCatList->count()))
    currentCatID++;
  
  fListBox->clear();
  fMemoList.first();
  while(fMemoList.current())
    {
      if(fMemoList.current()->getCat() == fCatList->currentItem())
	{
	  // List will delete it.
	  fListBox->insertItem(fMemoList.current()->getTitle());
	  fLookupTable[currentEntry++] = listIndex;
	}
      listIndex++;
      fMemoList.next();
    }
  fTextWidget->clear();
}

void
MemoWidget::slotShowMemo(int which)
{      
  disconnect(fTextWidget, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
  fTextWidget->deselect();
  fTextWidget->setText(fMemoList.at(fLookupTable[which])->text());
  connect(fTextWidget, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
}

void
MemoWidget::writeMemo(PilotMemo* which)
    {
    PilotDatabase* memoDB = KPilotLink::getPilotLink()->openLocalDatabase("MemoDB");
    PilotRecord* pilotRec = which->pack();
    memoDB->writeRecord(pilotRec);
    delete pilotRec;
    KPilotLink::getPilotLink()->closeDatabase(memoDB);
    }
    
void
MemoWidget::slotTextChanged()
{
  PilotMemo* currentMemo;

  if(fListBox->currentItem() >= 0)
    {
      currentMemo = fMemoList.at(fLookupTable[fListBox->currentItem()]);
      if(currentMemo->id() == 0x0)
	{
	  KMsgBox::message(0L, klocale->translate("Hot-Sync Required"), 
			   klocale->translate("Cannot edit new memo until \r\n Hot-Synced with pilot."), 
			   KMsgBox::STOP);
	  slotShowMemo(fListBox->currentItem());
	  return;
	}
      currentMemo->setText(fTextWidget->text().data());
      writeMemo(currentMemo);
    }
}

void
MemoWidget::slotImportMemo()
    {
    int i = 0;
    int nextChar;
    int currentCatID = -1;
  
    // If a category is deleted after others have been added, none of the
    // category numbers are changed.  So we need to find the category number
    // for this category.
    char notFound = 1;
    while (notFound)
      {
	currentCatID++;
	if (fMemoAppInfo.category.name[currentCatID])
	  notFound = strcmp(fMemoAppInfo.category.name[currentCatID],
			    fCatList->text(fCatList->currentItem()));
      }

    QString fileName = KFileDialog::getOpenFileName();
    if(fileName != NULL)
	{
	QFile importFile(fileName);
	if(importFile.open(IO_ReadOnly) == FALSE)
	    {
	    // show error!
	    return;
	    }
	char text[MemoWidget::MAX_MEMO_LEN];
	for(i = 0; (i < (MemoWidget::MAX_MEMO_LEN - 1)) && ((nextChar = importFile.getch()) != -1); i++)
	    text[i] = nextChar;
	text[i] = 0;
	PilotMemo* aMemo = new PilotMemo(text, 0, 0, currentCatID);
	fMemoList.append(aMemo);
	writeMemo(aMemo);
	updateWidget();
	}
    }

void
MemoWidget::slotExportMemo()
    {
    int item = fListBox->currentItem();
    const char* data;

    if(item == -1)
	return;
    item = fLookupTable[item];
    QString fileName = KFileDialog::getSaveFileName();
    if(fileName == 0L)
	return;

    data = fMemoList.at(item)->text();

    QFile outFile(fileName);
    if(outFile.open(IO_WriteOnly | IO_Truncate) == FALSE)
	{
	// show error!
	return;
	}
    QDataStream outStream(&outFile);
    outStream.writeRawBytes(data, strlen(data) + 1);
    outFile.close();
    }
